@@ -199,9 +199,10 @@ _UPGRADE_NOTIFY = True
 
 INCOMPLETE_READS_NUM_SUBCHUNKS = 8
 
-USER_AGENT = "{name}/{version} ({platform})".format(name=__name__,
+USER_AGENT = "{name}/{version} ({platform}) Python/{python_version}".format(name=__name__,
                                                     version=TOOLKIT_VERSION,
-                                                    platform=platform.platform())
+                                                    platform=platform.platform(),
+                                                    python_version=platform.python_version())
 _default_certs = requests.certs.where()
 _default_headers = requests.utils.default_headers()
 _default_timeout = urllib3.util.timeout.Timeout(connect=DEFAULT_TIMEOUT, read=DEFAULT_TIMEOUT)
@@ -314,6 +315,10 @@ _RETRYABLE_SOCKET_ERRORS = {
     errno.ECONNREFUSED  # A remote host refused to allow the network connection
 }
 
+_RETRYABLE_WITH_RESPONSE = (exceptions.ContentLengthError, BadStatusLine, exceptions.BadJSONInReply,
+                                         urllib3.exceptions.ProtocolError, exceptions.UrllibInternalError)
+if not USING_PYTHON2:
+    _RETRYABLE_WITH_RESPONSE += (ConnectionResetError,)
 
 def _is_retryable_exception(e):
     """Returns True if the exception is always safe to retry.
@@ -327,7 +332,9 @@ def _is_retryable_exception(e):
 
     """
     if isinstance(e, urllib3.exceptions.ProtocolError):
-        e = e.args[1]
+        return True
+    if not USING_PYTHON2 and isinstance(e, ConnectionResetError):
+        return True
     if isinstance(e, (socket.gaierror, socket.herror)):
         return True
     if isinstance(e, socket.error) and e.errno in _RETRYABLE_SOCKET_ERRORS:
@@ -352,7 +359,7 @@ def _extract_msg_from_last_exception():
         # '}')
         return last_error.error_message()
     else:
-        return traceback.format_exc().splitlines()[-1].strip()
+        return traceback.format_exception_only(last_exc_type, last_error)[-1].strip()
 
 
 def _calculate_retry_delay(response, num_attempts):
@@ -468,20 +475,6 @@ def _debug_print_response(debug_level, seq_num, time_started, req_id, response_s
               content_to_print,
               file=sys.stderr)
 
-
-def _test_tls_version():
-    tls12_check_script = os.path.join(os.getenv("DNANEXUS_HOME"), "build", "tls12check.py")
-    if not os.path.exists(tls12_check_script):
-        return
-
-    try:
-        subprocess.check_output(['python', tls12_check_script])
-    except subprocess.CalledProcessError as e:
-        if e.returncode == 1:
-            print (e.output)
-            raise exceptions.InvalidTLSProtocol
-
-
 def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                   timeout=DEFAULT_TIMEOUT,
                   use_compression=None, jsonify_data=True, want_full_response=False,
@@ -639,7 +632,6 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
 
                 response = pool_manager.request(_method, encoded_url, headers=_headers, body=body,
                                                 timeout=timeout, retries=False, **kwargs)
-
             except urllib3.exceptions.ClosedPoolError:
                 # If another thread closed the pool before the request was
                 # started, will throw ClosedPoolError
@@ -653,7 +645,7 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                and '_ARGCOMPLETE' not in os.environ):
                 logger.info(response.headers['x-upgrade-info'])
                 try:
-                    with file(_UPGRADE_NOTIFY, 'a'):
+                    with open(_UPGRADE_NOTIFY, 'a'):
                         os.utime(_UPGRADE_NOTIFY, None)
                 except:
                     pass
@@ -676,18 +668,17 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                     try:
                         error_class = getattr(exceptions, content["error"]["type"], exceptions.DXAPIError)
                     except (KeyError, AttributeError, TypeError):
-                        raise exceptions.HTTPError(response.status, content)
+                        raise exceptions.HTTPErrorWithContent("Appropriate error class not found. [HTTPCode=%s]" % response.status, content)
                     raise error_class(content, response.status, time_started, req_id)
                 else:
                     try:
                         content = response.data.decode('utf-8')
                     except AttributeError:
                         raise exceptions.UrllibInternalError("Content is none", response.status)
-                    raise exceptions.HTTPError("{} {} [Time={} RequestID={}]\n{}".format(response.status,
+                    raise exceptions.HTTPErrorWithContent("{} {} [Time={} RequestID={}]".format(response.status,
                                                                                          response.reason,
                                                                                          time_started,
-                                                                                         req_id,
-                                                                                         content))
+                                                                                         req_id), content.strip())
 
             if want_full_response:
                 return response
@@ -751,8 +742,7 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                     # BadStatusLine ---  server did not return anything
                     # BadJSONInReply --- server returned JSON that didn't parse properly
                     if (response is None
-                       or isinstance(e, (exceptions.ContentLengthError, BadStatusLine, exceptions.BadJSONInReply,
-                                         urllib3.exceptions.ProtocolError, exceptions.UrllibInternalError))):
+                       or isinstance(e, _RETRYABLE_WITH_RESPONSE)):
                         ok_to_retry = is_retryable
                     else:
                         ok_to_retry = 500 <= response.status < 600
@@ -761,7 +751,9 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                     if (response is not None
                        and response.status == 400 and is_retryable and method == 'PUT'
                        and isinstance(e, requests.exceptions.HTTPError)):
-                        if '<Code>RequestTimeout</Code>' in exception_msg:
+                        request_timeout_str = '<Code>RequestTimeout</Code>'
+                        if (request_timeout_str in exception_msg
+                            or (isinstance(e, exceptions.HTTPErrorWithContent) and request_timeout_str in e.content)):
                             logger.info("Retrying 400 HTTP error, due to slow data transfer. " +
                                         "Request Time=%f Request ID=%s", time_started, req_id)
                         else:
@@ -772,7 +764,6 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                     # Unprocessable entity, request has semantical errors
                     if response is not None and response.status == 422:
                         ok_to_retry = False
-
                 if ok_to_retry:
                     if rewind_input_buffer_offset is not None:
                         data.seek(rewind_input_buffer_offset)
@@ -786,8 +777,11 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
                         waiting_msg = 'Waiting %d seconds before retry %d of %d...' % (
                             delay, try_index + 1, max_retries)
 
-                    logger.warning("[%s] %s %s: %s. %s %s",
-                                   time.ctime(), method, _url, exception_msg, waiting_msg, range_str)
+                    log_msg = "[%s] %s %s: %s. %s %s" % (time.ctime(), method, _url, exception_msg, waiting_msg, range_str)
+                    if isinstance(e, exceptions.HTTPErrorWithContent):
+                        log_msg += "\n%s" % e.content
+
+                    logger.warning(log_msg)
                     time.sleep(delay)
                     try_index_including_503 += 1
                     if response is None or response.status != 503:
@@ -797,13 +791,10 @@ def DXHTTPRequest(resource, data, method='POST', headers=None, auth=True,
             # All retries have been exhausted OR the error is deemed not
             # retryable. Print the latest error and propagate it back to the caller.
             if not isinstance(e, exceptions.DXAPIError):
-                logger.error("[%s] %s %s: %s.", time.ctime(), method, _url, exception_msg)
-
-            if isinstance(e, urllib3.exceptions.ProtocolError) and \
-                'Connection reset by peer' in exception_msg:
-                # If the protocol error is 'connection reset by peer', most likely it is an
-                # error in the ssl handshake due to unsupported TLS protocol.
-                _test_tls_version()
+                log_msg = "[%s] %s %s: %s." % (time.ctime(), method, _url, exception_msg)
+                if isinstance(e, exceptions.HTTPErrorWithContent):
+                        log_msg += "\n%s" % e.content
+                logger.error(log_msg)
 
             # Retries have been exhausted, and we are unable to get a full
             # buffer from the data source. Raise a special exception.
