@@ -4,6 +4,7 @@ from .nextflow_utils import (get_template_dir, get_source_file_name, get_resourc
                                           get_importer_name, get_regional_options)
 import json
 import os
+from dxpy import TOOLKIT_VERSION
 from dxpy.compat import USING_PYTHON2,sys_encoding
 
 
@@ -39,8 +40,11 @@ def get_nextflow_dxapp(custom_inputs=None, name="", region="aws:us-east-1"):
     dxapp["title"] = name
     dxapp["summary"] = name
     dxapp["regionalOptions"] = get_regional_options(region)
+
+    # Record dxpy version used for this Nextflow build
+    dxapp["details"]["dxpyBuildVersion"] = TOOLKIT_VERSION
     if os.environ.get("DX_JOB_ID") is None or not is_importer_job():
-        dxapp["details"] = {"repository": "local"}
+        dxapp["details"]["repository"] = "local"
     return dxapp
 
 
@@ -62,34 +66,26 @@ def get_nextflow_src(custom_inputs=None, profile=None, resources_dir=None):
     with open(os.path.join(str(get_template_dir()), get_source_file_name()), 'r') as f:
         src = f.read()
 
-    required_runtime_params = ""
-    generate_runtime_config= ""
+    exclude_input_download=""
+    applet_runtime_params = ""
     for i in custom_inputs:
         value = "${%s}" % (i['name'])
         if i.get("class") == "file":
             value = "dx://${DX_WORKSPACE_ID}:/$(echo ${%s} | jq .[$dnanexus_link] -r | xargs -I {} dx describe {} --json | jq -r .name)" % i['name']
-        # optional inputs will be added to custom runtime config file
-        if "Nextflow pipeline optional" in i.get("help", ""):
-            if i.get("class") not in ("int","float","boolean"):
-                value = '\\"' + value + '\\"'
-            generate_runtime_config = generate_runtime_config + '''
-            if [ -n "$%s" ]; then
-                echo params.%s=%s >> nxf_runtime.config
-            fi    
-            '''% (i['name'], i['name'], value)
-        # required inputs need to be added as runtime pipeline params
-        else:
-            # required_inputs variable is initialized in the nextflow.sh script template
-            required_runtime_params = required_runtime_params + '''
-            if [ -n "${}" ]; then
-                required_inputs="${{required_inputs}} --{}={}" 
-            fi
-            '''.format(i['name'], i['name'], value)
+            exclude_input_download+="--except {} ".format(i['name'])
+
+        # applet_runtime_inputs variable is initialized in the nextflow.sh script template
+        applet_runtime_params = applet_runtime_params + '''
+        if [ -n "${}" ]; then
+            applet_runtime_inputs+=(--{} "{}")
+        fi
+        '''.format(i['name'], i['name'], value)
 
     profile_arg = "-profile {}".format(profile) if profile else ""
-    src = src.replace("@@GENERATE_RUNTIME_CONFIG@@", generate_runtime_config)
-    src = src.replace("@@REQUIRED_RUNTIME_PARAMS@@", required_runtime_params)
+    src = src.replace("@@APPLET_RUNTIME_PARAMS@@", applet_runtime_params)
     src = src.replace("@@PROFILE_ARG@@", profile_arg)
+    src = src.replace("@@EXCLUDE_INPUT_DOWNLOAD@@", exclude_input_download)
+    src = src.replace("@@DXPY_BUILD_VERSION@@", TOOLKIT_VERSION)
     if USING_PYTHON2:
         src = src.replace("@@RESOURCES_SUBPATH@@",
                       get_resources_subpath(resources_dir).encode(sys_encoding))
