@@ -1014,6 +1014,27 @@ def get_destination_region(destination):
         dest_project_id = dxpy.WORKSPACE_ID
     return dxpy.api.project_describe(dest_project_id, input_params={"fields": {"region": True}})["region"]
 
+def verify_nf_license(destination, extra_args):
+
+    def get_project_to_check():
+        # extra args overrides the destination argument
+        # so we're checking it first
+        if "project" in extra_args:
+            return extra_args["project"]
+        if destination:
+            dest_project_id, _, _ = parse_destination(destination)
+            # checkFeatureAccess is not implemented on the container
+            if dest_project_id.startswith("container-"):
+                dest_project_id = dxpy.PROJECT_CONTEXT_ID
+            return dest_project_id
+        else:
+            return dxpy.PROJECT_CONTEXT_ID
+
+    dest_project_to_check = get_project_to_check()
+    features = dxpy.DXHTTPRequest("/" + dest_project_to_check + "/checkFeatureAccess", {"features": ["dxNextflow"]}).get("features", {})
+    dx_nextflow_lic = features.get("dxNextflow", False)
+    if not dx_nextflow_lic:
+        raise dxpy.app_builder.AppBuilderException("PermissionDenied: billTo of the applet's destination project must have the dxNextflow feature enabled. For inquiries, please contact support@dnanexus.com")
 
 def _build_app(args, extra_args):
     """Builds an app or applet and returns the resulting executable ID
@@ -1025,6 +1046,10 @@ def _build_app(args, extra_args):
     resources_dir = None
     source_dir = args.src_dir
     worker_resources_subpath = ""  # no subpath, files will be saved to root directory by default.
+
+    if args.nextflow:
+        verify_nf_license(args.destination, extra_args)
+
     if args.nextflow and not args.repository:
         source_dir = prepare_nextflow(args.src_dir, args.profile, get_destination_region(args.destination))
         resources_dir = args.src_dir
