@@ -19,11 +19,14 @@
 from __future__ import print_function, unicode_literals, division, absolute_import
 from parameterized import parameterized
 
+import tempfile
+import shutil
 import os
 import sys
 import unittest
 import json
 from dxpy.nextflow.nextflow_templates import get_nextflow_src, get_nextflow_dxapp
+from dxpy.nextflow.nextflow_utils import get_template_dir
 
 import uuid
 from dxpy_testutil import (DXTestCase, DXTestCaseBuildNextflowApps, run, chdir)
@@ -32,7 +35,7 @@ from dxpy.compat import USING_PYTHON2, str, sys_encoding, open
 from dxpy.utils.resolver import ResolutionError
 import dxpy
 from dxpy.nextflow.nextflow_builder import prepare_custom_inputs
-
+from dxpy.nextflow.nextflow_utils import find_readme
 if USING_PYTHON2:
     spawn_extra_args = {}
 else:
@@ -40,7 +43,6 @@ else:
     spawn_extra_args = {"encoding": "utf-8"}
 
 
-default_input_len = 7
 input1 = {
     "class": "file",
     "name": "first_input",
@@ -68,18 +70,28 @@ input4 = {
 }
 
 
-class TestNextflowTemplates(DXTestCase):
+class TestNextflowUtils(DXTestCase):
 
-    def are_inputs_in_spec(self, inputSpec, inputs):
-        found = [False] * len(inputs)
-        input_names = [i["name"] for i in inputs]
-        input_pairs = dict(zip(input_names, found))
-        for i in inputSpec:
-            if i.get("name") in input_names:
-                if input_pairs.get(i.get("name")):
-                    raise Exception("Input was found twice!")
-                input_pairs[i.get("name")] = True
-        return all(value is True for value in input_pairs.values())
+    @parameterized.expand([
+        [None, 'file1.txt', 'file2.txt', 'main.nf'],
+        ['README.txt', 'README.txt', 'readme.txt', 'main.nf'],
+        ['README.md', 'README.md', 'main.nf'],
+        [None]
+    ])
+    def test_searching_readmes(self, expected, *file_list):
+        temp_dir = tempfile.mkdtemp()
+        try:
+            # creating a folder with files for find_readme
+            for filename in file_list:
+                file_path = os.path.join(temp_dir, filename)
+                open(file_path, 'a').close()
+            actual = find_readme(temp_dir)
+        finally:
+            shutil.rmtree(temp_dir)
+        assert actual == expected
+
+
+class TestNextflowTemplates(DXTestCase):
 
     def test_dxapp(self):
         dxapp = get_nextflow_dxapp()
@@ -92,12 +104,12 @@ class TestNextflowTemplates(DXTestCase):
         [input1, input2]
     ])
     def test_dxapp_custom_input(self, *inputs):
+        with open(os.path.join(str(get_template_dir()), 'dxapp.json'), 'r') as f:
+            default_dxapp = json.load(f)
+
         inputs = list(inputs)
         dxapp = get_nextflow_dxapp(custom_inputs=inputs)
-        self.assertTrue(self.are_inputs_in_spec(
-            dxapp.get("inputSpec"), inputs))
-        self.assertEqual(len(dxapp.get("inputSpec")),
-                         default_input_len + len(inputs))
+        self.assertEqual(dxapp.get("inputSpec"), inputs + default_dxapp.get("inputSpec"))
 
     @unittest.skipIf(USING_PYTHON2,
         'Skipping as the Nextflow template from which applets are built is for Py3 interpreter only')
@@ -117,31 +129,31 @@ class TestNextflowTemplates(DXTestCase):
     def test_src_inputs(self):
         '''
         Tests that code that handles custom nextflow input parameters (e.g. from nextflow schema) with different classes
-        are properly added in the applet source script. These input arguments should be
-        1) appended to nextflow cmd as runtime parameters
-        2) added to runtime configuration nxf_runtime.config if it is an optional param in nextflow_schema
+        are properly added in the applet source script. These input arguments should be appended to nextflow cmd as runtime parameters
         '''
         src = get_nextflow_src(custom_inputs=[input1, input2, input3, input4])
         # case 1: file input, need to convert from dnanexus link to its file path inside job workspace
         self.assertTrue("if [ -n \"${}\" ];".format(input1.get("name")) in src)
         value1 = 'dx://${DX_WORKSPACE_ID}:/$(echo ${%s} | jq .[$dnanexus_link] -r | xargs -I {} dx describe {} --json | jq -r .name)' % input1.get(
             "name")
-        self.assertTrue("--{}={}".format(input1.get("name"), value1) in src)
+        self.assertTrue("applet_runtime_inputs+=(--{} \"{}\")".format(input1.get("name"), value1) in src)
         # case 2: string input, need no conversion
         self.assertTrue("if [ -n \"${}\" ];".format(input2.get("name")) in src)
         value2 = '${%s}' % input2.get("name")
-        self.assertTrue("--{}={}".format(input2.get("name"), value2) in src)
-        # case 3: file input (nextflow pipeline optional), added to nxf_runtime.config
+        self.assertTrue("applet_runtime_inputs+=(--{} \"{}\")".format(input2.get("name"), value2) in src)
+        # case 3: file input (nextflow pipeline optional), same as case 1
         self.assertTrue("if [ -n \"${}\" ];".format(input3.get("name")) in src)
-        value3 = '\\"dx://${DX_WORKSPACE_ID}:/$(echo ${%s} | jq .[$dnanexus_link] -r | xargs -I {} dx describe {} --json | jq -r .name)\\"' % input3.get(
+        value3 = 'dx://${DX_WORKSPACE_ID}:/$(echo ${%s} | jq .[$dnanexus_link] -r | xargs -I {} dx describe {} --json | jq -r .name)' % input3.get(
             "name")
-        self.assertTrue("echo params.{}={} >> nxf_runtime.config".format(
-            input3.get("name"), value3) in src)
+        self.assertTrue("applet_runtime_inputs+=(--{} \"{}\")".format(input3.get("name"), value3) in src)
         # case 4: file input (nextflow pipeline required), same as case 1
         self.assertTrue("if [ -n \"${}\" ];".format(input4.get("name")) in src)
         value4 = 'dx://${DX_WORKSPACE_ID}:/$(echo ${%s} | jq .[$dnanexus_link] -r | xargs -I {} dx describe {} --json | jq -r .name)' % input4.get(
             "name")
-        self.assertTrue("--{}={}".format(input4.get("name"), value4) in src)
+        self.assertTrue("applet_runtime_inputs+=(--{} \"{}\")".format(input4.get("name"), value4) in src)
+        
+        self.assertTrue("dx-download-all-inputs --parallel --except {} --except {} --except {}".format(
+            input1.get("name"), input3.get("name"), input4.get("name")) in src)
 
     def test_prepare_inputs(self):
         inputs = prepare_custom_inputs(schema_file="./nextflow/schema2.json")
@@ -168,6 +180,7 @@ class TestNextflowTemplates(DXTestCase):
 class TestDXBuildNextflowApplet(DXTestCaseBuildNextflowApps):
 
     def test_dx_build_nextflow_default_metadata(self):
+        # Name of folder containing *.nf
         pipeline_name = "hello"
         applet_dir = self.write_nextflow_applet_directory(
             pipeline_name, existing_nf_file_path=self.base_nextflow_nf)
@@ -218,24 +231,25 @@ class TestDXBuildNextflowApplet(DXTestCaseBuildNextflowApps):
         self.assertEqual(app["name"], pipeline_name)
 
     def test_dx_build_nextflow_with_extra_args(self):
+        # Name of folder containing *.nf
         pipeline_name = "hello"
-        extra_args = '{"name": "testing_name_hello"}'
         applet_dir = self.write_nextflow_applet_directory(
             pipeline_name, existing_nf_file_path=self.base_nextflow_nf)
-        applet_id = json.loads(run(
-            "dx build --nextflow '{}' --json --extra-args '{}'".format(applet_dir, extra_args)))["id"]
-        app = dxpy.describe(applet_id)
-        self.assertEqual(app["name"], json.loads(extra_args)["name"])
-        self.assertEqual(app["title"], pipeline_name)
-        self.assertEqual(app["summary"], pipeline_name)
 
-        extra_args = '{"name": "new_name", "title": "new title"}'
+        # Override metadata values
+        extra_args = '{"name": "name-9Oxvx2tCZe", "title": "Title VsnhPeFBqt", "summary": "Summary 3E7fFfEXdB"}'
         applet_id = json.loads(run(
             "dx build --nextflow '{}' --json --extra-args '{}'".format(applet_dir, extra_args)))["id"]
-        app = dxpy.describe(applet_id)
-        self.assertEqual(app["name"], json.loads(extra_args)["name"])
-        self.assertEqual(app["title"], json.loads(extra_args)["title"])
-        self.assertEqual(app["summary"], pipeline_name)
+
+        applet = dxpy.DXApplet(applet_id)
+        desc = applet.describe()
+        self.assertEqual(desc["name"], json.loads(extra_args)["name"])
+        self.assertEqual(desc["title"], json.loads(extra_args)["title"])
+        self.assertEqual(desc["summary"], json.loads(extra_args)["summary"])
+
+        details = applet.get_details()
+        self.assertEqual(details["repository"], "local")
+
 
     @unittest.skipUnless(testutil.TEST_RUN_JOBS,
                          'skipping tests that would run jobs')
@@ -272,50 +286,21 @@ class TestDXBuildNextflowApplet(DXTestCaseBuildNextflowApps):
     @unittest.skipUnless(testutil.TEST_RUN_JOBS,
                          'skipping tests that would run jobs')
     def test_dx_build_nextflow_from_repository_with_extra_args(self):
-        pipeline_name = "hello"
         hello_repo_url = "https://github.com/nextflow-io/hello"
-        extra_args = '{"name": "new name", "title": "new title"}'
+
+        # Override metadata values
+        extra_args = '{"name": "name-l1DeZYnTyQ", "title": "Title KkWUaqpHh1", "summary": "Summary Yqf37VpDTY"}'
         applet_json = run("dx build --nextflow --repository '{}' --extra-args '{}' --brief".format(hello_repo_url, extra_args)).strip()
+
         applet_id = json.loads(applet_json).get("id")
         applet = dxpy.DXApplet(applet_id)
         desc = applet.describe()
-        self.assertEqual(desc["name"], "new name")
-        self.assertEqual(desc["title"], "new title")
-        self.assertEqual(desc["summary"], pipeline_name)
+        self.assertEqual(desc["name"], json.loads(extra_args)["name"])
+        self.assertEqual(desc["title"], json.loads(extra_args)["title"])
+        self.assertEqual(desc["summary"], json.loads(extra_args)["summary"])
 
-    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
-                         'skipping tests that would run jobs')
-    def test_dx_build_nextflow_with_publishDir(self):
-        pipeline_name = "cat_ls"
-        # extra_args = '{"name": "testing_cat_ls"}'
-        applet_dir = self.write_nextflow_applet_directory(
-            pipeline_name, nf_file_name="main.nf", existing_nf_file_path="nextflow/publishDir/main.nf")
-        applet_id = json.loads(run(
-            "dx build --nextflow '{}' --json".format(applet_dir)))["id"]
-        desc = dxpy.describe(applet_id)
-
-        # Run with "dx run".
-        dxfile = dxpy.upload_string("foo", name="foo.txt", folder="/a/b/c",
-                                    project=self.project, parents=True, wait_on_close=True)
-        inFile_path = "dx://{}:/a/b/c/foo.txt".format(self.project)
-        inFolder_path = "dx://{}:/a/".format(self.project)
-        outdir = "nxf_outdir"
-        pipeline_args = "'--outdir {} --inFile {} --inFolder {}'".format(
-            outdir, inFile_path, inFolder_path)
-
-        job_id = run(
-            "dx run {applet_id} -idebug=true -inextflow_pipeline_params={pipeline_args} --folder :/test-cat-ls/ -y --brief".format(
-                applet_id=applet_id, pipeline_args=pipeline_args)
-        ).strip()
-        job_handler = dxpy.DXJob(job_id)
-        job_handler.wait_on_done()
-        job_desc = dxpy.describe(job_id)
-
-        print(job_desc["output"])
-        self.assertEqual(len(job_desc["output"]["nextflow_log"]), 1)
-
-        # the output files will be: ls_folder.txt, cat_file.txt
-        self.assertEqual(len(job_desc["output"]["published_files"]), 2)
+        details = applet.get_details()
+        self.assertEqual(details["repository"], hello_repo_url)
 
     def test_dx_build_nextflow_with_destination(self):
         pipeline_name = "hello"
@@ -332,8 +317,9 @@ class TestDXBuildNextflowApplet(DXTestCaseBuildNextflowApps):
 
 class TestRunNextflowApplet(DXTestCaseBuildNextflowApps):
 
-    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
-                         'skipping tests that would run jobs')
+    # @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+    #                      'skipping tests that would run jobs')
+    @unittest.skip("skipping flaky test; to be fixed separately")
     def test_dx_run_retry_fail(self):
         pipeline_name = "retryMaxRetries"
         nextflow_file = "nextflow/RetryMaxRetries/main.nf"
@@ -363,12 +349,12 @@ class TestRunNextflowApplet(DXTestCaseBuildNextflowApps):
         applet = dxpy.DXApplet(applet_id)
 
         job = applet.run({
-                         "nextflow_pipeline_params": "--input 'Printed_test_message'",
+                         "nextflow_pipeline_params": "--input 'Printed test message'",
                          "nextflow_top_level_opts": "-quiet"
         })
 
         watched_run_output = run("dx watch {}".format(job.get_id()))
-        self.assertIn("Printed_test_message", watched_run_output)
+        self.assertIn("hello STDOUT Printed test message world!", watched_run_output)
         # Running with the -quiet option reduces the amount of log and the lines such as:
         # STDOUT Launching `/home/dnanexus/hello/main.nf` [run-c8804f26-2eac-48d2-9a1a-a707ad1189eb] DSL2 - revision: 72a5d52d07
         # are not printed
@@ -405,22 +391,127 @@ class TestRunNextflowApplet(DXTestCaseBuildNextflowApps):
                                 "extra_user_prop": "extra_value"}, new_job_desc["properties"])
         self.assertNotIn("nextflow_nonexistent_prop", new_job_desc["properties"])
 
-    # @unittest.skipUnless(testutil.TEST_RUN_JOBS,
-    #                      'skipping tests that would run jobs')
-    # def test_dx_run_nextflow_with_unsupported_runtime_opts(self):
-    #     pipeline_name = "hello"
-    #     applet_dir = self.write_nextflow_applet_directory(pipeline_name, existing_nf_file_path=self.base_nextflow_nf)
-    #     applet_id = json.loads(run("dx build --nextflow --json " + applet_dir))["id"]
-    #     applet = dxpy.DXApplet(applet_id)
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping tests that would run jobs')
+    def test_dx_run_nextflow_with_unsupported_runtime_opts(self):
+        pipeline_name = "hello"
+        applet_dir = self.write_nextflow_applet_directory(
+            pipeline_name, existing_nf_file_path=self.base_nextflow_nf)
+        applet_id = json.loads(
+            run("dx build --nextflow --json " + applet_dir))["id"]
+        applet = dxpy.DXApplet(applet_id)
 
-    #     job = applet.run({
-    #                      "nextflow_run_opts": "-w user_workdir",
-    #     })
+        job = applet.run({
+                         "nextflow_run_opts": "-w user_workdir",
+                         })
+        self.assertRaises(dxpy.exceptions.DXJobFailureError, job.wait_on_done)
+        desc = job.describe()
+        job_desc = dxpy.DXJob(job.get_id()).describe()
+        self.assertEqual(job_desc["failureReason"], "AppError")
+        self.assertIn("Please remove workDir specification",
+                      job_desc["failureMessage"])
 
-    #     job.wait_on_done()
-    #     job_desc = dxpy.describe(job.get_id())
-    #     self.assertEqual(job_desc["failureReason"], "AppError")
-    #     self.assertIn("Please remove workDir specification", job_desc["failureMessage"])
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping tests that would run jobs')
+    def test_dx_run_nextflow_with_publishDir(self):
+        pipeline_name = "cat_ls"
+        # extra_args = '{"name": "testing_cat_ls"}'
+        applet_dir = self.write_nextflow_applet_directory(
+            pipeline_name, nf_file_name="main.nf", existing_nf_file_path="nextflow/publishDir/main.nf")
+        applet_id = json.loads(run(
+            "dx build --nextflow '{}' --json".format(applet_dir)))["id"]
+        desc = dxpy.describe(applet_id)
+
+        # Run with "dx run".
+        dxfile = dxpy.upload_string("foo", name="foo.txt", folder="/a/b/c",
+                                    project=self.project, parents=True, wait_on_close=True)
+        inFile_path = "dx://{}:/a/b/c/foo.txt".format(self.project)
+        inFolder_path = "dx://{}:/a/".format(self.project)
+        outdir = "nxf_outdir"
+        pipeline_args = "'--outdir {} --inFile {} --inFolder {}'".format(
+            outdir, inFile_path, inFolder_path)
+
+        job_id = run(
+            "dx run {applet_id} -idebug=true -inextflow_pipeline_params={pipeline_args} --folder :/test-cat-ls/ -y --brief".format(
+                applet_id=applet_id, pipeline_args=pipeline_args)
+        ).strip()
+        job_handler = dxpy.DXJob(job_id)
+        job_handler.wait_on_done()
+        job_desc = dxpy.describe(job_id)
+
+        # the output files will be: ls_folder.txt, cat_file.txt
+        self.assertEqual(len(job_desc["output"]["published_files"]), 2)
+
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping tests that would run jobs')
+    def test_dx_run_override_profile(self):
+        pipeline_name = "profile_test"
+
+        applet_dir = self.write_nextflow_applet_directory_from_folder(pipeline_name, "nextflow/profile/")
+        applet_id = json.loads(run(
+            "dx build --nextflow --profile test '{}' --json".format(applet_dir)))["id"]
+        job_id = run(
+            "dx run {applet_id} -y -inextflow_run_opts=\"-profile second\" --brief".format(applet_id=applet_id)
+        ).strip()
+
+        job_handler = dxpy.DXJob(job_id)
+        job_handler.wait_on_done()
+        watched_run_output = run("dx watch {} --no-follow".format(job_id))
+
+        self.assertTrue("second_config world!" in watched_run_output, "second_config world! test was NOT found in the job log of {job_id}".format(job_id=job_id))
+        self.assertTrue("test_config world!" not in watched_run_output, "test_config world! test was found in the job log of {job_id}, but it should have been overriden".format(job_id=job_id))
+
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping tests that would run jobs')
+    def test_dx_run_nextflow_with_soft_conf_files(self):
+        pipeline_name = "print_env"
+        applet_dir = self.write_nextflow_applet_directory(
+            pipeline_name,  existing_nf_file_path="nextflow/print_env_nextflow_soft_confs/main.nf")
+        applet_id = json.loads(run(
+            "dx build --nextflow '{}' --json".format(applet_dir)))["id"]
+
+        # Run with "dx run".
+        first_config = dxpy.upload_local_file("nextflow/print_env_nextflow_soft_confs/first.config", project=self.project, wait_on_close=True).get_id()
+        second_config = dxpy.upload_local_file("nextflow/print_env_nextflow_soft_confs/second.config", project=self.project, wait_on_close=True).get_id()
+
+        job_id = run(
+            "dx run {applet_id} -idebug=true -inextflow_soft_confs={first_config} -inextflow_soft_confs={second_config} --brief -y".format(
+                applet_id=applet_id, first_config=first_config, second_config=second_config)
+        ).strip()
+        job_handler = dxpy.DXJob(job_id)
+        job_handler.wait_on_done()
+        watched_run_output = run("dx watch {} --no-follow".format(job_id))
+        self.assertTrue("-c /home/dnanexus/in/nextflow_soft_confs/0/first.config -c /home/dnanexus/in/nextflow_soft_confs/1/second.config" in watched_run_output)
+        # env var ALPHA specified in first.config and second.config
+        # the value in second.config overrides the one in first.config
+        self.assertTrue("The env var ALPHA is: runtime alpha 2" in watched_run_output)
+        # env var BETA specified in first.config only
+        self.assertTrue("The env var BETA is: runtime beta 1" in watched_run_output)
+
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping tests that would run jobs')
+    def test_dx_run_nextflow_with_runtime_param_file(self):
+        pipeline_name = "print_params"
+        applet_dir = self.write_nextflow_applet_directory(
+            pipeline_name,  existing_nf_file_path="nextflow/print_param_nextflow_params_file/main.nf")
+        applet_id = json.loads(run(
+            "dx build --nextflow '{}' --json".format(applet_dir)))["id"]
+
+        # Run with "dx run".
+        params_file = dxpy.upload_local_file("nextflow/print_param_nextflow_params_file/params_file.yml", project=self.project, wait_on_close=True).get_id()
+
+        job_id = run(
+            "dx run {applet_id} -idebug=true -inextflow_params_file={params_file} -inextflow_pipeline_params=\"--BETA 'CLI beta'\" --brief -y".format(
+                applet_id=applet_id, params_file=params_file)
+        ).strip()
+        job_handler = dxpy.DXJob(job_id)
+        job_handler.wait_on_done()
+        watched_run_output = run("dx watch {} --no-follow".format(job_id))
+
+        self.assertTrue("-params-file /home/dnanexus/in/nextflow_params_file/params_file.yml" in watched_run_output)
+        # precedence of the input parameter values: nextflow_params_file < nextflow_pipeline_params < other applet runtime inputs parsed from nextflow schema
+        self.assertTrue("The parameter ALPHA is: param file alpha" in watched_run_output)
+        self.assertTrue("The parameter BETA is: CLI beta" in watched_run_output)
 
 if __name__ == '__main__':
     if 'DXTEST_FULL' not in os.environ:
